from .Builders import generate_xml, generate_invoice, write_xml_from_string, generate_facturx
from .Validators import xml_check_xsd, xml_check_schematron
__version__ = "0.1.4"
__all__ = [
    "generate_xml",
    "generate_invoice",
    "write_xml_from_string",
    "xml_check_xsd",
    "xml_check_schematron",
    "generate_facturx"
]


from pathlib import Path
from saxonche import PySaxonProcessor
import xml.etree.ElementTree as ET

def xml_check_schematron(xml_path: str , xslt_path: str = "xslt/EN16931-CII-validation.xslt"):
    """
    Applies Schematron validation on an XML file and returns the errors.
    :param xml_path: Path to the XML file to validate.
    :param xslt_path: Path to the XSLT file containing the validation rules.
    :return: Validation report containing the errors.
    """

    NS = {"svrl": "http://purl.oclc.org/dsdl/svrl"}
    base = Path(__file__).parent.resolve()
    xslt_path = (base / xslt_path).as_posix()

    with PySaxonProcessor(license=False) as proc:
        xslt = proc.new_xslt30_processor()
        #xslt.set_cwd(base.as_posix())
        result = xslt.transform_to_string(
            source_file=xml_path,
            stylesheet_file=xslt_path
        )

    # Parse the result (SVRL XML string)
    root = ET.fromstring(result)
    fails = root.findall(".//svrl:failed-assert", NS)

    return {f"{len(fails)} fails": create_report(fails, NS)}

def create_report(fails:list, NS):
    """
    Displays errors from schematron validation
    :param fails: Validation report containing the errors.
    :return: None
    """
    list = []

    for fa in fails:
        rid = fa.get("id")
        loc = fa.get("location")
        msg = fa.findtext("svrl:text", default="", namespaces=NS)
        fail = {rid : msg, "Location": loc}
        list.append(fail)
    return list






sample_raw_data = {
    "company": {
        "name": "ALCHIMIE DATA SOLUTIONS",
        "address": [
            "10 rue du SQL",
            "75010 PARIS 10 - France",
        ],
        "siren" : "123456789"
    },

    "partner" : {
        "name": "Mon client préféré",
        "address": [
            "6 Rue de Excel",
            "59300 Roubaix - France",
        ],
        "siren" : "987654321"},

    "invoice": {
        "name" : "1234",
        "order": "S00028",
        "date": "20260113",
        "expiration": "20260212",
        "salesperson": "Antoine Ducoulombier",
        "reference" : "CAC-123456789",
        "description" : ["Période du 01/01/2025 au 31/12/2025."]
    },
    "invoice_header":
        [
            {
            "tva_code":10,
            "tva_texte": "TVA 10%",
            "amount": 18.00
            },
            {
            "tva_code":20,
            "tva_texte": "TVA 20%",
            "amount": 108.00
        }],
    "invoice_lines": [
        {
            "product_code": "A3",
            "product_description": "Booking fee",
            "product_type": "LB",
            "quantity": 10,
            "unit_price": 50.00,
            "tva_code":20,
            "tva_text": "TVA 20%",
            "amount": 108.00
        }
    ],
    "footer": {
        "left": "+33688305889",
        "middle" : "contact@alchimiedatasolutions.com",
        "right": "http://www.alchimiedatasolutions.com",
        "bottom": "123456789"
    },

    "other": {

        "terms_url": "https://mon_site_web.com"
    }
}



from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.enums import TA_RIGHT
from reportlab.lib import colors

def get_styles():
    styles = getSampleStyleSheet()
    styles.add(ParagraphStyle(
        name="Right",
        alignment=TA_RIGHT
    ))
    styles.add(ParagraphStyle(
        name="Titles",
        fontSize=16,
        spaceAfter=12,
        bold=True
    ))

    styles.add(ParagraphStyle(
    name="TempTitle",
    parent=styles["Normal"],
    fontSize=22,
    textColor=colors.black,
    fontName="Helvetica"
))

    return styles
